# Author: Jan Vcelak <jvcelak@redhat.com>

require "yaml"

class BaseConfig
	def initialize(options = nil)
		@options = self.class.get_defaults
		merge! options unless options.nil?
	end

	def self.get_defaults
		{}.clone
	end

	def self.load_file(filename)
		options = YAML.load_file filename
		self.new options
	end

	def [](path)
		index_get @options, path
	end

	def []=(path, value)
		index_set @options, path, value
	end

	def merge!(other_options)
		merge_recursive @options, other_options
	end

	private

	def index_get(hash, path)
		if path.nil?
			return hash
		elsif not hash.kind_of? Hash
			return nil
		end

		(option, next_path) = path.split(".", 2)
		if hash.include? option
			return index_get hash[option], next_path
		else
			return nil
		end
	end

	def index_set(hash, path, value)
		(option, next_path) = path.split(".", 2)

		if not hash.include? option
			if next_path.nil?
				hash[option] = value
			else
				hash[option] = {}
				index_set hash[option], next_path, value
			end
		elsif next_path.nil?
			hash[option] = value
		elsif hash[option].kind_of? Hash
			index_set hash[option], next_path, value
		else
			raise Exception.new "Parent node is not a Hash."
		end
	end

	def merge_recursive(first, second)
		second.each do |key, value|
			if first.include? key and first[key].kind_of? Hash and value.kind_of? Hash
				merge_recursive(first[key], value)
			else
				first[key] = value
			end
		end
	end
end
